@@ -9,115 +9,119 @@ utils::globalVariables(c("engine", "mx", "plots"))
 #'
 #' @param options named \code{list} of \code{knitr} options. Supported options are \code{echo}, \code{eval}, \code{include} and \code{output.var}. To change the output format of the Maxima engine set the option \code{maxima.options(engine.format)} to either \code{"linear"} (default), \code{"ascii"}, \code{"latex"} or \code{"mathml"}.
 #'
-#' @import knitr
-#' @importFrom utils tail
-#'
-#' @return This functions prints the resulting output from maxima together with it's code
+#' @return This functions prints the resulting output from maxima together with it's cod
 maxima.engine <- function(options) {
   maxima.engine.start()
-  code <- options$code
-  ov <- !is.null(varname <- options$output.var)
+
+  out_code <- options$code
+  ccode <- character(0)
   output.data <- list()
-  code <- code[nchar(code) > 0]
-  cmds <- gather(code)
+  out_res <- character(0)
   ll <- list()
-  ccode <- character()
+
+  ov <- !is.null(varname <- options$output.var)
+  cmds <- dissect_chunk(out_code)
+
   if (options$eval) {
     for (i in 1:length(cmds)) {
-      pc <- paste0(code[cmds[[i]]], collapse = "\n")
+      pc <- paste0(out_code[cmds[[i]]], collapse = "\n")
       # if plotting command, then it needs to end with ";"
       if (grepl(pattern = "^(?:plot|draw)(?:2d|3d)?\\([[:print:]|[:space:]]+\\)[[:space:]]*\\$$", x = pc)) {
         pc <- gsub(pattern = "\\$", replacement = ";", x = pc)
       }
       tt <- maxima.env$mx$get(pc)
-      ccode <- append(ccode, iprint(tt))
-      if (!attr(tt, "suppressed")) {
-        if (ov) output.data[[substring(attr(tt, "output.label"), 2L)]] <- attr(tt, "parsed")
-        if (options$echo) {
-          ll <- append(ll, list(structure(list(src = ccode), class = "source")))
-        }
 
-        if (grepl(pattern = "^(?:plot|draw)(?:2d|3d)?\\([[:print:]|[:space:]]+\\)[[:space:]]*;", x = pc)) {
+      attr(tt, "from_engine") <- TRUE
+
+      out_code[cmds[[i]]] <- unlist(strsplit(x = iprint(tt), split = '\n', fixed = TRUE))
+      ccode <- append(ccode, unlist(strsplit(x = iprint(tt), split = '\n', fixed = TRUE)))
+      if (!attr(tt, "suppressed")) {
+        ll <- append(ll, list(structure(list(src = ccode), class = "source")))
+        if (ov) 
+          output.data[[substring(attr(tt, "output.label"), 2L)]] <- attr(tt, "parsed")
+
+        # +++ FIGURE OUTPUT +++
+        if (grepl(pattern = "^(?:plot|draw)(?:2d|3d)?\\([[:print:]|[:space:]]+\\)[[:space:]]*;", x = pc) &
+            !is.null(knitr::all_labels(engine == "maxima"))) {
           tt$wol$ascii <- paste0(tt$wol$ascii, collapse = "")
           pm <- regexec(pattern = "\\[?([[:graph:]]*/?\\.?(?:plot|draw)(?:2d|3d)?-[a-z0-9]+\\.(?:png|pdf))\\]$", text = tt$wol$ascii)
           pm <- trim(unlist(regmatches(m = pm, x = tt$wol$ascii))[2])
 
-          # it's possible the image has not yet been written to disk
+          # possibly image not yet written to disk
           pm <- retry_include_graphics(pm)
           ll <- append(ll, list(pm))
+          out_res <- c(out_res, list(pm))
           maxima.env$plots <- append(maxima.env$plots, normalizePath(pm, mustWork = FALSE))
         } else {
-          ll <- append(ll, engine_print(tt))
+        # +++ TEXT OUTPUT +++
+          ll <- append(ll, ttt <- print(tt))
+          out_res <- c(out_res, ttt)
         }
-        ccode <- character()
+        ccode <- character(0)
       }
     }
 
     if (ov) {
-      assign(varname, output.data, envir = knit_global())
-    }
-
-    if (length(ccode)) {
-      if (options$echo) {
-        ll <- append(ll, list(structure(list(src = ccode), class = "source")))
-      }
-    }
-  } else {
-    if (options$echo) {
-      ll <- append(ll, list(structure(list(src = code), class = "source")))
+      assign(varname, output.data, envir = knitr::knit_global())
     }
   }
 
-  if (last_label(options$label)) {
+  # called_from_fn("knit") not needed since engine is ALWAYS called from knit
+  if (last_label(options$label) & called_from_fn("knit")) {
     maxima.engine.stop()
   }
 
-  engine_output(opts_current$merge(list(results = maxima.options$engine.results)), out = ll)
+  # special handling for RStudio
+  if(is_interactive()) {
+    knitr::engine_output(knitr::opts_current$merge(list(engine = 'maxima',
+                                                        lang = 'maxima',
+                                                        results = 'markup')),
+                         code = NULL,
+                         out = out_res)
+  } else {
+    knitr::engine_output(knitr::opts_current$merge(list(engine = 'maxima',
+                                                        lang = 'maxima',
+                                                        results = maxima.options$engine.results)), 
+                         out = ll)
+  }
 }
 
 maxima.engine.start <- function() {
-  if (!exists("mx", envir = maxima.env)) {
-    maxima.env$mx <- RMaxima$new(
-      display = maxima.options$display,
-      preload = maxima.options$preload[knitr::is_latex_output() + 1]
-    )
-    maxima.env$plots <- character()
+  if(is.null(knitr::all_labels(engine == "maxima"))) {
+    maxima.env$mx <- maxima.env$maxima
+  } else {
+    if (!exists("mx", envir = maxima.env)) {
+      maxima.env$mx <- RMaxima$new(
+                                   display = maxima.options$display,
+                                   preload = maxima.options$preload[knitr::is_latex_output() + 1]
+      )
+      maxima.env$plots <- character()
+    }
   }
 }
 
 maxima.engine.stop <- function() {
-  maxima.env$mx$stop()
-  e <- sys.frame(which = 1)
-  do.call("on.exit", list(quote(if (exists("maxima.env")) file.remove(maxima.env$plots)), add = TRUE), envir = e)
-  do.call("on.exit", list(quote(if (exists("maxima.env")) rm(plots, envir = maxima.env)), add = TRUE), envir = e)
-  rm(mx, envir = maxima.env)
+  if(!is.null(knitr::all_labels(engine == "maxima"))) {
+    maxima.env$mx$stop()
+    e <- sys.frame(which = 1)
+    do.call("on.exit", list(quote(if (exists("maxima.env")) file.remove(maxima.env$plots)), add = TRUE), envir = e)
+    do.call("on.exit", list(quote(if (exists("maxima.env")) rm(plots, envir = maxima.env)), add = TRUE), envir = e)
+    rm(mx, envir = maxima.env)
+  }
 }
 
 last_label <- function(label = knitr::opts_current$get("label")) {
   # if (knitr:::child_mode()) return(FALSE)
-  if (knitr::opts_knit$get("child")) {
+  if(knitr::opts_knit$get("child")) {
     return(FALSE)
   }
+
+  if(is_interactive()) {
+    return(TRUE)
+  }
+
   labels <- knitr::all_labels(engine == "maxima")
-  tail(labels, 1) == label
-}
-
-engine_print <- function(x) {
-  pp <- switch(maxima.options$engine.label + 1,
-    paste0(c(x[["wol"]][[maxima.options$engine.format]], ""), collapse = "\n"),
-    paste0(c(x[["wtl"]][[maxima.options$engine.format]], ""), collapse = "\n")
-  )
-  if (is_html_output()) {
-    pp <- gsub(pattern = "\\\\%", replacement = "%", x = pp)
-  }
-  pp
-}
-
-is_html_output <- function() {
-  if (is.null(p <- knitr::opts_knit$get("rmarkdown.pandoc.to"))) {
-    return(FALSE)
-  }
-  knitr::is_html_output() & p == "html"
+  utils::tail(labels, 1) == label
 }
 
 #' @describeIn maxima.engine This function can be used to insert maxima outputs as inline.
@@ -138,4 +142,3 @@ maxima.inline <- function(command) {
     paste0(c(x[["wtl"]][[maxima.options$inline.format]], ""), collapse = "\n")
   )
 }
-
